@@ -15,9 +15,7 @@
 
 import copy
 from distutils.version import StrictVersion
-import functools
 import hashlib
-import json
 import logging
 import os
 import socket
@@ -26,6 +24,8 @@ import textwrap
 import time
 
 from keystoneauth1 import adapter
+from keystoneauth1 import exceptions as kexc
+from oslo_serialization import jsonutils
 from oslo_utils import strutils
 import requests
 import six
@@ -34,8 +34,6 @@ import six.moves.urllib.parse as urlparse
 
 from ironicclient.common import filecache
 from ironicclient.common.i18n import _
-from ironicclient.common.i18n import _LE
-from ironicclient.common.i18n import _LW
 from ironicclient import exc
 
 
@@ -45,7 +43,8 @@ from ironicclient import exc
 #             http://specs.openstack.org/openstack/ironic-specs/specs/kilo/api-microversions.html # noqa
 #             for full details.
 DEFAULT_VER = '1.9'
-
+LAST_KNOWN_API_VERSION = 46
+LATEST_VERSION = '1.{}'.format(LAST_KNOWN_API_VERSION)
 
 LOG = logging.getLogger(__name__)
 USER_AGENT = 'python-ironicclient'
@@ -65,28 +64,36 @@ SUPPORTED_ENDPOINT_SCHEME = ('http', 'https')
 
 def _trim_endpoint_api_version(url):
     """Trim API version and trailing slash from endpoint."""
-    return url.rstrip('/').rstrip(API_VERSION)
+    return url.rstrip('/').rstrip(API_VERSION).rstrip('/')
 
 
 def _extract_error_json(body):
     """Return  error_message from the HTTP response body."""
-    error_json = {}
     try:
-        body_json = json.loads(body)
-        if 'error_message' in body_json:
-            raw_msg = body_json['error_message']
-            error_json = json.loads(raw_msg)
+        body_json = jsonutils.loads(body)
     except ValueError:
-        pass
+        return {}
 
-    return error_json
+    if 'error_message' not in body_json:
+        return {}
+
+    try:
+        error_json = jsonutils.loads(body_json['error_message'])
+    except ValueError:
+        return body_json
+
+    err_msg = (error_json.get('faultstring') or error_json.get('description'))
+    if err_msg:
+        body_json['error_message'] = err_msg
+
+    return body_json
 
 
-def get_server(endpoint):
-    """Extract and return the server & port that we're connecting to."""
-    if endpoint is None:
+def get_server(url):
+    """Extract and return the server & port."""
+    if url is None:
         return None, None
-    parts = urlparse.urlparse(endpoint)
+    parts = urlparse.urlparse(url)
     return parts.hostname, str(parts.port)
 
 
@@ -96,9 +103,46 @@ class VersionNegotiationMixin(object):
 
         Assumption: Called after receiving a 406 error when doing a request.
 
-        param conn: A connection object
-        param resp: The response object from http request
+        :param conn: A connection object
+        :param resp: The response object from http request
         """
+        def _query_server(conn):
+            if (self.os_ironic_api_version and
+                    not isinstance(self.os_ironic_api_version, list) and
+                    self.os_ironic_api_version != 'latest'):
+                base_version = ("/v%s" %
+                                str(self.os_ironic_api_version).split('.')[0])
+            else:
+                base_version = API_VERSION
+            return self._make_simple_request(conn, 'GET', base_version)
+
+        version_overridden = False
+
+        if (resp and hasattr(resp, 'request') and
+                hasattr(resp.request, 'headers')):
+            orig_hdr = resp.request.headers
+            # Get the version of the client's last request and fallback
+            # to the default for things like unit tests to not cause
+            # migraines.
+            req_api_ver = orig_hdr.get('X-OpenStack-Ironic-API-Version',
+                                       self.os_ironic_api_version)
+        else:
+            req_api_ver = self.os_ironic_api_version
+        if (resp and req_api_ver != self.os_ironic_api_version and
+                self.api_version_select_state == 'negotiated'):
+            # If we have a non-standard api version on the request,
+            # but we think we've negotiated, then the call was overridden.
+            # We should report the error with the called version
+            requested_version = req_api_ver
+            # And then we shouldn't save the newly negotiated
+            # version of this negotiation because we have been
+            # overridden a request.
+            version_overridden = True
+        else:
+            requested_version = self.os_ironic_api_version
+
+        if not resp:
+            resp = _query_server(conn)
         if self.api_version_select_state not in API_VERSION_SELECTED_STATES:
             raise RuntimeError(
                 _('Error: self.api_version_select_state should be one of the '
@@ -111,36 +155,83 @@ class VersionNegotiationMixin(object):
         # the supported version range
         if not max_ver:
             LOG.debug('No version header in response, requesting from server')
-            if self.os_ironic_api_version:
-                base_version = ("/v%s" %
-                                str(self.os_ironic_api_version).split('.')[0])
-            else:
-                base_version = API_VERSION
-            resp = self._make_simple_request(conn, 'GET', base_version)
+            resp = _query_server(conn)
             min_ver, max_ver = self._parse_version_headers(resp)
+        # Reset the maximum version that we permit
+        if StrictVersion(max_ver) > StrictVersion(LATEST_VERSION):
+            LOG.debug("Remote API version %(max_ver)s is greater than the "
+                      "version supported by ironicclient. Maximum available "
+                      "version is %(client_ver)s",
+                      {'max_ver': max_ver,
+                       'client_ver': LATEST_VERSION})
+            max_ver = LATEST_VERSION
+
         # If the user requested an explicit version or we have negotiated a
         # version and still failing then error now.  The server could
         # support the version requested but the requested operation may not
         # be supported by the requested version.
-        if self.api_version_select_state == 'user':
+        # TODO(TheJulia): We should break this method into several parts,
+        # such as a sanity check/error method.
+        if ((self.api_version_select_state == 'user' and
+                not self._must_negotiate_version()) or
+                (self.api_version_select_state == 'negotiated' and
+                    version_overridden)):
             raise exc.UnsupportedVersion(textwrap.fill(
                 _("Requested API version %(req)s is not supported by the "
-                  "server or the requested operation is not supported by the "
-                  "requested version.  Supported version range is %(min)s to "
+                  "server, client, or the requested operation is not "
+                  "supported by the requested version. "
+                  "Supported version range is %(min)s to "
                   "%(max)s")
-                % {'req': self.os_ironic_api_version,
+                % {'req': requested_version,
                    'min': min_ver, 'max': max_ver}))
-        if self.api_version_select_state == 'negotiated':
+        if (self.api_version_select_state == 'negotiated'):
             raise exc.UnsupportedVersion(textwrap.fill(
-                _("No API version was specified and the requested operation "
+                _("No API version was specified or the requested operation "
                   "was not supported by the client's negotiated API version "
                   "%(req)s.  Supported version range is: %(min)s to %(max)s")
-                % {'req': self.os_ironic_api_version,
+                % {'req': requested_version,
                    'min': min_ver, 'max': max_ver}))
 
-        negotiated_ver = str(min(StrictVersion(self.os_ironic_api_version),
-                                 StrictVersion(max_ver)))
-        if negotiated_ver < min_ver:
+        if isinstance(requested_version, six.string_types):
+            if requested_version == 'latest':
+                negotiated_ver = max_ver
+            else:
+                negotiated_ver = str(
+                    min(StrictVersion(requested_version),
+                        StrictVersion(max_ver)))
+
+        elif isinstance(requested_version, list):
+            if 'latest' in requested_version:
+                raise ValueError(textwrap.fill(
+                    _("The 'latest' API version can not be requested "
+                      "in a list of versions. Please explicitly request "
+                      "'latest' or request only versios between "
+                      "%(min)s to %(max)s")
+                    % {'min': min_ver, 'max': max_ver}))
+
+            versions = []
+            for version in requested_version:
+                if min_ver <= StrictVersion(version) <= max_ver:
+                    versions.append(StrictVersion(version))
+            if versions:
+                negotiated_ver = str(max(versions))
+            else:
+                raise exc.UnsupportedVersion(textwrap.fill(
+                    _("Requested API version specified and the requested "
+                      "operation was not supported by the client's "
+                      "requested API version %(req)s.  Supported "
+                      "version range is: %(min)s to %(max)s")
+                    % {'req': requested_version,
+                       'min': min_ver, 'max': max_ver}))
+
+        else:
+            raise ValueError(textwrap.fill(
+                _("Requested API version %(req)s type is unsupported. "
+                  "Valid types are Strings such as '1.1', 'latest' "
+                  "or a list of string values representing API versions.")
+                % {'req': requested_version}))
+
+        if StrictVersion(negotiated_ver) < StrictVersion(min_ver):
             negotiated_ver = min_ver
         # server handles microversions, but doesn't support
         # the requested version, so try a negotiated version
@@ -149,7 +240,10 @@ class VersionNegotiationMixin(object):
         LOG.debug('Negotiated API version is %s', negotiated_ver)
 
         # Cache the negotiated version for this server
-        host, port = get_server(self.endpoint)
+        # TODO(vdrok): get rid of self.endpoint attribute in Stein
+        endpoint_override = (getattr(self, 'endpoint_override', None) or
+                             getattr(self, 'endpoint', None))
+        host, port = get_server(endpoint_override)
         filecache.save_data(host=host, port=port, data=negotiated_ver)
 
         return negotiated_ver
@@ -169,14 +263,18 @@ class VersionNegotiationMixin(object):
         # NOTE(jlvillal): Declared for unit testing purposes
         raise NotImplementedError()
 
+    def _must_negotiate_version(self):
+        return (self.api_version_select_state == 'user' and
+                (self.os_ironic_api_version == 'latest' or
+                 isinstance(self.os_ironic_api_version, list)))
 
 _RETRY_EXCEPTIONS = (exc.Conflict, exc.ServiceUnavailable,
-                     exc.ConnectionRefused)
+                     exc.ConnectionRefused, kexc.RetriableConnectionFailure)
 
 
 def with_retries(func):
     """Wrapper for _http_request adding support for retries."""
-    @functools.wraps(func)
+    @six.wraps(func)
     def wrapper(self, url, method, **kwargs):
         if self.conflict_max_retries is None:
             self.conflict_max_retries = DEFAULT_MAX_RETRIES
@@ -188,8 +286,8 @@ def with_retries(func):
             try:
                 return func(self, url, method, **kwargs)
             except _RETRY_EXCEPTIONS as error:
-                msg = (_LE("Error contacting Ironic server: %(error)s. "
-                           "Attempt %(attempt)d of %(total)d") %
+                msg = ("Error contacting Ironic server: %(error)s. "
+                       "Attempt %(attempt)d of %(total)d" %
                        {'attempt': attempt,
                         'total': num_attempts,
                         'error': error})
@@ -206,6 +304,8 @@ def with_retries(func):
 class HTTPClient(VersionNegotiationMixin):
 
     def __init__(self, endpoint, **kwargs):
+        LOG.warning('HTTPClient class is deprecated and will be removed '
+                    'in Stein release, please use SessionClient instead.')
         self.endpoint = endpoint
         self.endpoint_trimmed = _trim_endpoint_api_version(endpoint)
         self.auth_token = kwargs.get('token')
@@ -275,7 +375,7 @@ class HTTPClient(VersionNegotiationMixin):
             body = strutils.mask_password(kwargs['body'])
             curl.append('-d \'%s\'' % body)
 
-        curl.append(urlparse.urljoin(self.endpoint_trimmed, url))
+        curl.append(self._make_connection_url(url))
         LOG.debug(' '.join(curl))
 
     @staticmethod
@@ -293,7 +393,10 @@ class HTTPClient(VersionNegotiationMixin):
         LOG.debug('\n'.join(dump))
 
     def _make_connection_url(self, url):
-        return urlparse.urljoin(self.endpoint_trimmed, url)
+        # NOTE(pas-ha) we already stripped trailing / from endpoint_trimmed
+        if not url.startswith('/'):
+            url = '/' + url
+        return self.endpoint_trimmed + url
 
     def _parse_version_headers(self, resp):
         return self._generic_parse_version_headers(resp.headers.get)
@@ -308,6 +411,10 @@ class HTTPClient(VersionNegotiationMixin):
         Wrapper around request.Session.request to handle tasks such
         as setting headers and error handling.
         """
+        # NOTE(TheJulia): self.os_ironic_api_version is reset in
+        # the self.negotiate_version() call if negotiation occurs.
+        if self.os_ironic_api_version and self._must_negotiate_version():
+            self.negotiate_version(self.session, None)
         # Copy the kwargs so we can reuse the original in case of redirects
         kwargs['headers'] = copy.deepcopy(kwargs.get('headers', {}))
         kwargs['headers'].setdefault('User-Agent', USER_AGENT)
@@ -334,7 +441,7 @@ class HTTPClient(VersionNegotiationMixin):
 
             # TODO(deva): implement graceful client downgrade when connecting
             # to servers that did not support microversions. Details here:
-            # http://specs.openstack.org/openstack/ironic-specs/specs/kilo/api-microversions.html#use-case-3b-new-client-communicating-with-a-old-ironic-user-specified  # noqa
+            # https://specs.openstack.org/openstack/ironic-specs/specs/kilo-implemented/api-microversions.html#use-case-3b-new-client-communicating-with-a-old-ironic-user-specified  # noqa
 
             if resp.status_code == http_client.NOT_ACCEPTABLE:
                 negotiated_ver = self.negotiate_version(self.session, resp)
@@ -353,21 +460,20 @@ class HTTPClient(VersionNegotiationMixin):
 
             raise exc.ConnectionRefused(message)
 
-        body_iter = resp.iter_content(chunk_size=CHUNKSIZE)
-
-        # Read body into string if it isn't obviously image data
         body_str = None
-        if resp.headers.get('Content-Type') != 'application/octet-stream':
-            body_str = ''.join([chunk for chunk in body_iter])
+        if resp.headers.get('Content-Type') == 'application/octet-stream':
+            body_iter = resp.iter_content(chunk_size=CHUNKSIZE)
+            self.log_http_response(resp)
+        else:
+            # Read body into string if it isn't obviously image data
+            body_str = resp.text
             self.log_http_response(resp, body_str)
             body_iter = six.StringIO(body_str)
-        else:
-            self.log_http_response(resp)
 
         if resp.status_code >= http_client.BAD_REQUEST:
             error_json = _extract_error_json(body_str)
             raise exc.from_response(
-                resp, error_json.get('faultstring'),
+                resp, error_json.get('error_message'),
                 error_json.get('debuginfo'), method, url)
         elif resp.status_code in (http_client.MOVED_PERMANENTLY,
                                   http_client.FOUND,
@@ -385,7 +491,7 @@ class HTTPClient(VersionNegotiationMixin):
         kwargs['headers'].setdefault('Accept', 'application/json')
 
         if 'body' in kwargs:
-            kwargs['body'] = json.dumps(kwargs['body'])
+            kwargs['body'] = jsonutils.dump_as_bytes(kwargs['body'])
 
         resp, body_iter = self._http_request(url, method, **kwargs)
         content_type = resp.headers.get('Content-Type')
@@ -398,9 +504,9 @@ class HTTPClient(VersionNegotiationMixin):
         if 'application/json' in content_type:
             body = ''.join([chunk for chunk in body_iter])
             try:
-                body = json.loads(body)
+                body = jsonutils.loads(body)
             except ValueError:
-                LOG.error(_LE('Could not decode response body as JSON'))
+                LOG.error('Could not decode response body as JSON')
         else:
             body = None
 
@@ -414,7 +520,7 @@ class HTTPClient(VersionNegotiationMixin):
 
 
 class VerifiedHTTPSConnection(six.moves.http_client.HTTPSConnection):
-    """httplib-compatibile connection using client-side SSL authentication
+    """httplib-compatible connection using client-side SSL authentication
 
     :see http://code.activestate.com/recipes/
             577548-https-httplib-client-connection-with-certificate-v/
@@ -485,13 +591,19 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
                  api_version_select_state,
                  max_retries,
                  retry_interval,
-                 endpoint,
+                 endpoint=None,
                  **kwargs):
         self.os_ironic_api_version = os_ironic_api_version
         self.api_version_select_state = api_version_select_state
         self.conflict_max_retries = max_retries
         self.conflict_retry_interval = retry_interval
-        self.endpoint = endpoint
+        # TODO(vdrok): remove this conditional in Stein
+        if endpoint and not kwargs.get('endpoint_override'):
+            LOG.warning('Passing "endpoint" argument to SessionClient '
+                        'constructor is deprecated, such possibility will be '
+                        'removed in Stein. Please use "endpoint_override" '
+                        'instead.')
+            self.endpoint = endpoint
 
         super(SessionClient, self).__init__(**kwargs)
 
@@ -499,13 +611,33 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
         return self._generic_parse_version_headers(resp.headers.get)
 
     def _make_simple_request(self, conn, method, url):
+        endpoint_filter = {
+            'interface': self.interface,
+            'service_type': self.service_type,
+            'region_name': self.region_name
+        }
+
         # NOTE: conn is self.session for this class
-        return conn.request(url, method, raise_exc=False)
+        return conn.request(url, method, raise_exc=False,
+                            user_agent=USER_AGENT,
+                            endpoint_filter=endpoint_filter)
 
     @with_retries
     def _http_request(self, url, method, **kwargs):
+
+        # NOTE(TheJulia): self.os_ironic_api_version is reset in
+        # the self.negotiate_version() call if negotiation occurs.
+        if self.os_ironic_api_version and self._must_negotiate_version():
+            self.negotiate_version(self.session, None)
+
         kwargs.setdefault('user_agent', USER_AGENT)
         kwargs.setdefault('auth', self.auth)
+        if isinstance(self.endpoint_override, six.string_types):
+            kwargs.setdefault(
+                'endpoint_override',
+                _trim_endpoint_api_version(self.endpoint_override)
+            )
+
         if getattr(self, 'os_ironic_api_version', None):
             kwargs['headers'].setdefault('X-OpenStack-Ironic-API-Version',
                                          self.os_ironic_api_version)
@@ -524,7 +656,7 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
             return self._http_request(url, method, **kwargs)
         if resp.status_code >= http_client.BAD_REQUEST:
             error_json = _extract_error_json(resp.content)
-            raise exc.from_response(resp, error_json.get('faultstring'),
+            raise exc.from_response(resp, error_json.get('error_message'),
                                     error_json.get('debuginfo'), method, url)
         elif resp.status_code in (http_client.MOVED_PERMANENTLY,
                                   http_client.FOUND, http_client.USE_PROXY):
@@ -541,7 +673,7 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
         kwargs['headers'].setdefault('Accept', 'application/json')
 
         if 'body' in kwargs:
-            kwargs['data'] = json.dumps(kwargs.pop('body'))
+            kwargs['data'] = jsonutils.dump_as_bytes(kwargs.pop('body'))
 
         resp = self._http_request(url, method, **kwargs)
         body = resp.content
@@ -554,7 +686,7 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
             try:
                 body = resp.json()
             except ValueError:
-                LOG.error(_LE('Could not decode response body as JSON'))
+                LOG.error('Could not decode response body as JSON')
         else:
             body = None
 
@@ -567,8 +699,7 @@ class SessionClient(VersionNegotiationMixin, adapter.LegacyJsonAdapter):
         return self._http_request(url, method, **kwargs)
 
 
-def _construct_http_client(endpoint=None,
-                           session=None,
+def _construct_http_client(session=None,
                            token=None,
                            auth_ref=None,
                            os_ironic_api_version=DEFAULT_VER,
@@ -584,8 +715,8 @@ def _construct_http_client(endpoint=None,
     if session:
         kwargs.setdefault('service_type', 'baremetal')
         kwargs.setdefault('user_agent', 'python-ironicclient')
-        kwargs.setdefault('interface', kwargs.pop('endpoint_type', None))
-        kwargs.setdefault('endpoint_override', endpoint)
+        kwargs.setdefault('interface', kwargs.pop('endpoint_type',
+                                                  'publicURL'))
 
         ignored = {'token': token,
                    'auth_ref': auth_ref,
@@ -598,8 +729,8 @@ def _construct_http_client(endpoint=None,
         dvars = [k for k, v in ignored.items() if v]
 
         if dvars:
-            LOG.warning(_LW('The following arguments are ignored when using '
-                            'the session to construct a client: %s'),
+            LOG.warning('The following arguments are ignored when using '
+                        'the session to construct a client: %s',
                         ', '.join(dvars))
 
         return SessionClient(session=session,
@@ -607,12 +738,13 @@ def _construct_http_client(endpoint=None,
                              api_version_select_state=api_version_select_state,
                              max_retries=max_retries,
                              retry_interval=retry_interval,
-                             endpoint=endpoint,
                              **kwargs)
     else:
+        endpoint = None
         if kwargs:
-            LOG.warning(_LW('The following arguments are being ignored when '
-                            'constructing the client: %s'), ', '.join(kwargs))
+            endpoint = kwargs.pop('endpoint_override', None)
+            LOG.warning('The following arguments are being ignored when '
+                        'constructing the client: %s'), ', '.join(kwargs)
 
         return HTTPClient(endpoint=endpoint,
                           token=token,
