@@ -16,7 +16,6 @@
 from __future__ import print_function
 
 import argparse
-import base64
 import contextlib
 import gzip
 import json
@@ -26,8 +25,9 @@ import subprocess
 import sys
 import tempfile
 
-from oslo_utils import importutils
+from oslo_serialization import base64
 from oslo_utils import strutils
+import six
 
 from ironicclient.common.i18n import _
 from ironicclient import exc
@@ -35,9 +35,7 @@ from ironicclient import exc
 
 class HelpFormatter(argparse.HelpFormatter):
     def start_section(self, heading):
-        # Title-case the headings
-        heading = '%s%s' % (heading[0].upper(), heading[1:])
-        super(HelpFormatter, self).start_section(heading)
+        super(HelpFormatter, self).start_section(heading.capitalize())
 
 
 def define_command(subparsers, command, callback, cmd_mapper):
@@ -58,8 +56,13 @@ def define_command(subparsers, command, callback, cmd_mapper):
     subparser.add_argument('-h', '--help', action='help',
                            help=argparse.SUPPRESS)
     cmd_mapper[command] = subparser
+    required_args = subparser.add_argument_group(_("Required arguments"))
+
     for (args, kwargs) in arguments:
-        subparser.add_argument(*args, **kwargs)
+        if kwargs.get('required'):
+            required_args.add_argument(*args, **kwargs)
+        else:
+            subparser.add_argument(*args, **kwargs)
     subparser.set_defaults(func=callback)
 
 
@@ -73,13 +76,6 @@ def define_commands_from_module(subparsers, command_module, cmd_mapper):
         define_command(subparsers, command, callback, cmd_mapper)
 
 
-def import_versioned_module(version, submodule=None):
-    module = 'ironicclient.v%s' % version
-    if submodule:
-        module = '.'.join((module, submodule))
-    return importutils.import_module(module)
-
-
 def split_and_deserialize(string):
     """Split and try to JSON deserialize a string.
 
@@ -150,6 +146,27 @@ def args_array_to_patch(op, attributes):
     return patch
 
 
+def convert_list_props_to_comma_separated(data, props=None):
+    """Convert the list-type properties to comma-separated strings
+
+    :param data: the input dict object.
+    :param props: the properties whose values will be converted.
+        Default to None to convert all list-type properties of the input.
+    :returns: the result dict instance.
+    """
+    result = dict(data)
+
+    if props is None:
+        props = data.keys()
+
+    for prop in props:
+        val = data.get(prop, None)
+        if isinstance(val, list):
+            result[prop] = ', '.join(map(six.text_type, val))
+
+    return result
+
+
 def common_params_for_list(args, fields, field_labels):
     """Generate 'params' dict that is common for every 'list' command.
 
@@ -198,7 +215,7 @@ def common_params_for_list(args, fields, field_labels):
 
 
 def common_filters(marker=None, limit=None, sort_key=None, sort_dir=None,
-                   fields=None):
+                   fields=None, detail=False):
     """Generate common filters for any list request.
 
     :param marker: entity ID from which to start returning entities.
@@ -207,6 +224,9 @@ def common_filters(marker=None, limit=None, sort_key=None, sort_dir=None,
     :param sort_dir: direction of sorting: 'asc' or 'desc'.
     :param fields: a list with a specified set of fields of the resource
                    to be returned.
+    :param detail: Boolean, True to return detailed information. This parameter
+                   can be used for resources which accept 'detail' as a URL
+                   parameter.
     :returns: list of string filters.
     """
     filters = []
@@ -220,6 +240,8 @@ def common_filters(marker=None, limit=None, sort_key=None, sort_dir=None,
         filters.append('sort_dir=%s' % sort_dir)
     if fields is not None:
         filters.append('fields=%s' % ','.join(fields))
+    if detail:
+        filters.append('detail=True')
     return filters
 
 
@@ -270,12 +292,11 @@ def make_configdrive(path):
 
             # Compress file
             tmpfile.seek(0)
-            g = gzip.GzipFile(fileobj=tmpzipfile, mode='wb')
-            shutil.copyfileobj(tmpfile, g)
-            g.close()
+            with gzip.GzipFile(fileobj=tmpzipfile, mode='wb') as gz_file:
+                shutil.copyfileobj(tmpfile, gz_file)
 
             tmpzipfile.seek(0)
-            return base64.b64encode(tmpzipfile.read())
+            return base64.encode_as_bytes(tmpzipfile.read())
 
 
 def check_empty_arg(arg, arg_descriptor):
