@@ -14,6 +14,7 @@
 
 import os
 
+import six
 import six.moves.configparser as config_parser
 from tempest.lib.cli import base
 from tempest.lib.common.utils import data_utils
@@ -30,6 +31,8 @@ class FunctionalTestBase(base.ClientTestBase):
     def setUp(self):
         super(FunctionalTestBase, self).setUp()
         self.client = self._get_clients()
+        # NOTE(kromanenko) set ironic api version for portgroups
+        self.pg_api_ver = '--ironic-api-version 1.25'
 
     def _get_clients(self):
         # NOTE(aarefiev): {toxinidir} is a current working directory, so
@@ -41,7 +44,7 @@ class FunctionalTestBase(base.ClientTestBase):
             client = base.CLIClient(cli_dir=cli_dir,
                                     username=config['os_username'],
                                     password=config['os_password'],
-                                    tenant_name=config['os_tenant_name'],
+                                    tenant_name=config['os_project_name'],
                                     uri=config['os_auth_url'])
             for keystone_object in 'user', 'project':
                 domain_attr = 'os_%s_domain_id' % keystone_object
@@ -49,16 +52,18 @@ class FunctionalTestBase(base.ClientTestBase):
                     setattr(self, domain_attr, config[domain_attr])
         else:
             self.ironic_url = config['ironic_url']
-            self.os_auth_token = config['os_auth_token']
             client = base.CLIClient(cli_dir=cli_dir,
-                                    ironic_url=self.ironic_url,
-                                    os_auth_token=self.os_auth_token)
+                                    ironic_url=self.ironic_url)
         return client
 
     def _get_config(self):
         config_file = os.environ.get('IRONICCLIENT_TEST_CONFIG',
                                      DEFAULT_CONFIG_FILE)
-        config = config_parser.SafeConfigParser()
+        # SafeConfigParser was deprecated in Python 3.2
+        if six.PY3:
+            config = config_parser.ConfigParser()
+        else:
+            config = config_parser.SafeConfigParser()
         if not config.read(config_file):
             self.skipTest('Skipping, no test config found @ %s' % config_file)
         try:
@@ -74,11 +79,12 @@ class FunctionalTestBase(base.ClientTestBase):
         keystone_v3_conf_settings = []
         if auth_strategy == 'keystone':
             conf_settings += ['os_auth_url', 'os_username',
-                              'os_password', 'os_tenant_name']
+                              'os_password', 'os_project_name']
             keystone_v3_conf_settings += ['os_user_domain_id',
-                                          'os_project_domain_id']
+                                          'os_project_domain_id',
+                                          'os_identity_api_version']
         else:
-            conf_settings += ['os_auth_token', 'ironic_url']
+            conf_settings += ['ironic_url']
 
         cli_flags = {}
         missing = []
@@ -111,27 +117,40 @@ class FunctionalTestBase(base.ClientTestBase):
         :param params: optional positional args to use
         :type params: string
         """
-        flags = ('--os_auth_token %(token)s --ironic_url %(url)s %(flags)s'
+        flags = ('--os-endpoint %(url)s %(flags)s'
                  %
-                 {'token': self.os_auth_token,
-                  'url': self.ironic_url,
+                 {'url': self.ironic_url,
                   'flags': flags})
         return base.execute(cmd, action, flags, params,
                             cli_dir=self.client.cli_dir)
 
-    def _ironic(self, action, flags='', params=''):
+    def _ironic(self, action, cmd='ironic', flags='', params='',
+                merge_stderr=False):
         """Execute ironic command for the given action.
 
         :param action: the cli command to run using Ironic
         :type action: string
+        :param cmd: the base of cli command to run
+        :type action: string
         :param flags: any optional cli flags to use
         :type flags: string
         :param params: any optional positional args to use
         :type params: string
+        :param merge_stderr: whether to merge stderr into the result
+        :type merge_stderr: bool
         """
-        flags += ' --os-endpoint-type publicURL'
-        if hasattr(self, 'os_auth_token'):
-            return self._cmd_no_auth('ironic', action, flags, params)
+        if cmd == 'openstack':
+            config = self._get_config()
+            id_api_version = config.get('os_identity_api_version')
+            if id_api_version:
+                flags += ' --os-identity-api-version {}'.format(id_api_version)
+        else:
+            flags += ' --os-endpoint-type publicURL'
+
+        if hasattr(self, 'ironic_url'):
+            if cmd == 'openstack':
+                flags += ' --os-auth-type none'
+            return self._cmd_no_auth(cmd, action, flags, params)
         else:
             for keystone_object in 'user', 'project':
                 domain_attr = 'os_%s_domain_id' % keystone_object
@@ -140,8 +159,8 @@ class FunctionalTestBase(base.ClientTestBase):
                         'ks_obj': keystone_object,
                         'value': getattr(self, domain_attr)
                     }
-            return self.client.cmd_with_auth('ironic',
-                                             action, flags, params)
+            return self.client.cmd_with_auth(
+                cmd, action, flags, params, merge_stderr=merge_stderr)
 
     def ironic(self, action, flags='', params='', parse=True):
         """Return parsed list of dicts with basic item info.
@@ -191,12 +210,15 @@ class FunctionalTestBase(base.ClientTestBase):
         """Delete node method works only with fake driver.
 
         :param node_id: node uuid
+        :raises: CommandFailed exception when command fails to delete a node
         """
         node_list = self.list_nodes()
 
         if utils.get_object(node_list, node_id):
             node_show = self.show_node(node_id)
-            if node_show['provision_state'] != 'available':
+            if node_show['provision_state'] not in ('available',
+                                                    'manageable',
+                                                    'enroll'):
                 self.ironic('node-set-provision-state',
                             params='{0} deleted'.format(node_id))
             if node_show['power_state'] not in ('None', 'off'):
@@ -209,7 +231,7 @@ class FunctionalTestBase(base.ClientTestBase):
                 self.fail('Ironic node {0} has not been deleted!'
                           .format(node_id))
 
-    def create_node(self, driver='fake', params=''):
+    def create_node(self, driver='fake-hardware', params=''):
         node = self.ironic('node-create',
                            params='--driver {0} {1}'.format(driver, params))
 
@@ -246,9 +268,10 @@ class FunctionalTestBase(base.ClientTestBase):
             'node-set-maintenance',
             params='{0} {1} {2}'.format(node_id, maintenance_mode, params))
 
-    def set_node_power_state(self, node_id, power_state):
+    def set_node_power_state(self, node_id, power_state, params=''):
         self.ironic('node-set-power-state',
-                    params='{0} {1}'.format(node_id, power_state))
+                    params='{0} {1} {2}'
+                    .format(node_id, power_state, params))
 
     def set_node_provision_state(self, node_id, provision_state, params=''):
         self.ironic('node-set-provision-state',
@@ -327,12 +350,14 @@ class FunctionalTestBase(base.ClientTestBase):
     def create_port(self,
                     node_id,
                     mac_address=None,
+                    flags='',
                     params=''):
 
         if mac_address is None:
             mac_address = data_utils.rand_mac_address()
 
         port = self.ironic('port-create',
+                           flags=flags,
                            params='--address {0} --node {1} {2}'
                            .format(mac_address, node_id, params))
         if not port:
@@ -352,8 +377,64 @@ class FunctionalTestBase(base.ClientTestBase):
         port_list = self.list_ports()
         return [x['UUID'] for x in port_list]
 
-    def update_port(self, port_id, operation, params=''):
+    def update_port(self, port_id, operation, flags='', params=''):
         updated_port = self.ironic('port-update',
+                                   flags=flags,
                                    params='{0} {1} {2}'
                                    .format(port_id, operation, params))
         return utils.get_dict_from_output(updated_port)
+
+    def create_portgroup(self, node_id, params=''):
+        """Create a new portgroup."""
+        portgroup = self.ironic('portgroup-create',
+                                flags=self.pg_api_ver,
+                                params='--node {0} {1}'
+                                .format(node_id, params))
+        if not portgroup:
+            self.fail('Ironic portgroup failed to create!')
+        portgroup = utils.get_dict_from_output(portgroup)
+        self.addCleanup(self.delete_portgroup, portgroup['uuid'],
+                        ignore_exceptions=True)
+        return portgroup
+
+    def delete_portgroup(self, portgroup_id, ignore_exceptions=False):
+        """Delete a port group."""
+        try:
+            self.ironic('portgroup-delete',
+                        flags=self.pg_api_ver,
+                        params=portgroup_id)
+        except exceptions.CommandFailed:
+            if not ignore_exceptions:
+                raise
+
+    def list_portgroups(self, params=''):
+        """List the port groups."""
+        return self.ironic('portgroup-list',
+                           flags=self.pg_api_ver,
+                           params=params)
+
+    def show_portgroup(self, portgroup_id, params=''):
+        """Show detailed information about a port group."""
+        portgroup_show = self.ironic('portgroup-show',
+                                     flags=self.pg_api_ver,
+                                     params='{0} {1}'
+                                     .format(portgroup_id, params))
+        return utils.get_dict_from_output(portgroup_show)
+
+    def update_portgroup(self, portgroup_id, op, params=''):
+        """Update information about a port group."""
+        updated_portgroup = self.ironic('portgroup-update',
+                                        flags=self.pg_api_ver,
+                                        params='{0} {1} {2}'
+                                        .format(portgroup_id, op, params))
+        return utils.get_dict_from_output(updated_portgroup)
+
+    def get_portgroup_uuids_from_portgroup_list(self):
+        """Get UUIDs from list of port groups."""
+        portgroup_list = self.list_portgroups()
+        return [x['UUID'] for x in portgroup_list]
+
+    def portgroup_port_list(self, portgroup_id, params=''):
+        """List the ports associated with a port group."""
+        return self.ironic('portgroup-port-list', flags=self.pg_api_ver,
+                           params='{0} {1}'.format(portgroup_id, params))
