@@ -13,10 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
 import time
 
 import mock
+from oslo_serialization import jsonutils
+import requests
 import six
 from six.moves import http_client
 
@@ -34,15 +35,17 @@ DEFAULT_HOST = 'localhost'
 DEFAULT_PORT = '1234'
 
 
-def _get_error_body(faultstring=None, debuginfo=None):
-    error_body = {
-        'faultstring': faultstring,
-        'debuginfo': debuginfo
-    }
-    raw_error_body = json.dumps(error_body)
+def _get_error_body(faultstring=None, debuginfo=None, description=None):
+    if description:
+        error_body = {'description': description}
+    else:
+        error_body = {
+            'faultstring': faultstring,
+            'debuginfo': debuginfo
+        }
+    raw_error_body = jsonutils.dump_as_bytes(error_body)
     body = {'error_message': raw_error_body}
-    raw_body = json.dumps(body)
-    return raw_body
+    return jsonutils.dumps(body)
 
 
 def _session_client(**kwargs):
@@ -180,6 +183,164 @@ class VersionNegotiationMixinTest(utils.BaseTestCase):
         self.assertEqual(1, mock_pvh.call_count)
         self.assertEqual(0, mock_save_data.call_count)
 
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_strict_version_comparison(self, mock_pvh,
+                                                         mock_save_data):
+        # Test version comparison with StrictVersion
+        max_ver = '1.10'
+        mock_pvh.return_value = ('1.2', max_ver)
+        mock_conn = mock.MagicMock()
+        self.test_object.os_ironic_api_version = '1.10'
+        result = self.test_object.negotiate_version(mock_conn, self.response)
+        self.assertEqual(max_ver, result)
+        self.assertEqual(1, mock_pvh.call_count)
+        host, port = http.get_server(self.test_object.endpoint)
+        mock_save_data.assert_called_once_with(host=host, port=port,
+                                               data=max_ver)
+
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_make_simple_request',
+                       autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_server_user_latest(
+            self, mock_pvh, mock_msr, mock_save_data):
+        # have to retry with simple get
+        mock_pvh.side_effect = iter([(None, None), ('1.1', '1.99')])
+        mock_conn = mock.MagicMock()
+        self.test_object.api_version_select_state = 'user'
+        self.test_object.os_ironic_api_version = 'latest'
+        result = self.test_object.negotiate_version(mock_conn, None)
+        self.assertEqual(http.LATEST_VERSION, result)
+        self.assertEqual('negotiated',
+                         self.test_object.api_version_select_state)
+        self.assertEqual(http.LATEST_VERSION,
+                         self.test_object.os_ironic_api_version)
+
+        self.assertTrue(mock_msr.called)
+        self.assertEqual(2, mock_pvh.call_count)
+        self.assertEqual(1, mock_save_data.call_count)
+
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_make_simple_request',
+                       autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_server_user_list(
+            self, mock_pvh, mock_msr, mock_save_data):
+        # have to retry with simple get
+        mock_pvh.side_effect = [(None, None), ('1.1', '1.26')]
+        mock_conn = mock.MagicMock()
+        self.test_object.api_version_select_state = 'user'
+        self.test_object.os_ironic_api_version = ['1.1', '1.6', '1.25',
+                                                  '1.26', '1.26.1', '1.27',
+                                                  '1.30']
+        result = self.test_object.negotiate_version(mock_conn, self.response)
+        self.assertEqual('1.26', result)
+        self.assertEqual('negotiated',
+                         self.test_object.api_version_select_state)
+        self.assertEqual('1.26',
+                         self.test_object.os_ironic_api_version)
+
+        self.assertTrue(mock_msr.called)
+        self.assertEqual(2, mock_pvh.call_count)
+        self.assertEqual(1, mock_save_data.call_count)
+
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_make_simple_request',
+                       autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_server_user_list_fails_nomatch(
+            self, mock_pvh, mock_msr, mock_save_data):
+        # have to retry with simple get
+        mock_pvh.side_effect = iter([(None, None), ('1.2', '1.26')])
+        mock_conn = mock.MagicMock()
+        self.test_object.api_version_select_state = 'user'
+        self.test_object.os_ironic_api_version = ['1.39', '1.1']
+        self.assertRaises(
+            exc.UnsupportedVersion,
+            self.test_object.negotiate_version,
+            mock_conn, self.response)
+        self.assertEqual('user',
+                         self.test_object.api_version_select_state)
+        self.assertEqual(['1.39', '1.1'],
+                         self.test_object.os_ironic_api_version)
+        self.assertEqual(2, mock_pvh.call_count)
+        self.assertEqual(0, mock_save_data.call_count)
+
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_make_simple_request',
+                       autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_server_user_list_single_value(
+            self, mock_pvh, mock_msr, mock_save_data):
+        # have to retry with simple get
+        mock_pvh.side_effect = iter([(None, None), ('1.1', '1.26')])
+        mock_conn = mock.MagicMock()
+        self.test_object.api_version_select_state = 'user'
+        # NOTE(TheJulia): Lets test this value explicitly because the
+        # minor number is actually the same.
+        self.test_object.os_ironic_api_version = ['1.01']
+        result = self.test_object.negotiate_version(mock_conn, None)
+        self.assertEqual('1.1', result)
+        self.assertEqual('negotiated',
+                         self.test_object.api_version_select_state)
+        self.assertEqual('1.1',
+                         self.test_object.os_ironic_api_version)
+        self.assertTrue(mock_msr.called)
+        self.assertEqual(2, mock_pvh.call_count)
+        self.assertEqual(1, mock_save_data.call_count)
+
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_make_simple_request',
+                       autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_server_user_list_fails_latest(
+            self, mock_pvh, mock_msr, mock_save_data):
+        # have to retry with simple get
+        mock_pvh.side_effect = iter([(None, None), ('1.1', '1.2')])
+        mock_conn = mock.MagicMock()
+        self.test_object.api_version_select_state = 'user'
+        self.test_object.os_ironic_api_version = ['1.01', 'latest']
+        self.assertRaises(
+            ValueError,
+            self.test_object.negotiate_version,
+            mock_conn, self.response)
+        self.assertEqual('user',
+                         self.test_object.api_version_select_state)
+        self.assertEqual(['1.01', 'latest'],
+                         self.test_object.os_ironic_api_version)
+        self.assertEqual(2, mock_pvh.call_count)
+        self.assertEqual(0, mock_save_data.call_count)
+
+    @mock.patch.object(filecache, 'save_data', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_make_simple_request',
+                       autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, '_parse_version_headers',
+                       autospec=True)
+    def test_negotiate_version_explicit_version_request(
+            self, mock_pvh, mock_msr, mock_save_data):
+        mock_pvh.side_effect = iter([(None, None), ('1.1', '1.99')])
+        mock_conn = mock.MagicMock()
+        self.test_object.api_version_select_state = 'negotiated'
+        self.test_object.os_ironic_api_version = '1.30'
+        req_header = {'X-OpenStack-Ironic-API-Version': '1.29'}
+        response = utils.FakeResponse(
+            {}, status=http_client.NOT_ACCEPTABLE,
+            request_headers=req_header)
+        self.assertRaisesRegexp(exc.UnsupportedVersion,
+                                ".*is not supported by the server.*",
+                                self.test_object.negotiate_version,
+                                mock_conn, response)
+        self.assertTrue(mock_msr.called)
+        self.assertEqual(2, mock_pvh.call_count)
+        self.assertFalse(mock_save_data.called)
+
     def test_get_server(self):
         host = 'ironic-host'
         port = '6385'
@@ -189,6 +350,11 @@ class VersionNegotiationMixinTest(utils.BaseTestCase):
 
 class HttpClientTest(utils.BaseTestCase):
 
+    @mock.patch.object(http.LOG, 'warning', autospec=True)
+    def test_http_client_deprecation(self, log_mock):
+        http.HTTPClient('http://localhost')
+        self.assertIn('deprecated', log_mock.call_args[0][0])
+
     def test_url_generation_trailing_slash_in_base(self):
         client = http.HTTPClient('http://localhost/')
         url = client._make_connection_url('/v1/resources')
@@ -204,12 +370,24 @@ class HttpClientTest(utils.BaseTestCase):
         url = client._make_connection_url('v1/resources')
         self.assertEqual('http://localhost/v1/resources', url)
 
+    def test_server_https_request_with_application_octet_stream(self):
+        client = http.HTTPClient('https://localhost/')
+        client.session = utils.mockSession(
+            {'Content-Type': 'application/octet-stream'},
+            "Body",
+            version=1,
+            status_code=http_client.OK)
+
+        response, body = client.json_request('GET', '/v1/resources')
+        self.assertEqual(client.session.request.return_value, response)
+        self.assertIsNone(body)
+
     def test_server_exception_empty_body(self):
         error_body = _get_error_body()
         client = http.HTTPClient('http://localhost/')
-        client.session = utils.FakeSession(
+        client.session = utils.mockSession(
             {'Content-Type': 'application/json'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.INTERNAL_SERVER_ERROR)
 
@@ -221,9 +399,9 @@ class HttpClientTest(utils.BaseTestCase):
         error_msg = 'test error msg'
         error_body = _get_error_body(error_msg)
         client = http.HTTPClient('http://localhost/')
-        client.session = utils.FakeSession(
+        client.session = utils.mockSession(
             {'Content-Type': 'application/json'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.INTERNAL_SERVER_ERROR)
 
@@ -231,11 +409,25 @@ class HttpClientTest(utils.BaseTestCase):
                           client.json_request,
                           'GET', '/v1/resources')
 
+    def test_server_exception_description_only(self):
+        error_msg = 'test error msg'
+        error_body = _get_error_body(description=error_msg)
+        client = http.HTTPClient('http://localhost/')
+        client.session = utils.mockSession(
+            {'Content-Type': 'application/json'},
+            error_body,
+            version=1,
+            status_code=http_client.BAD_REQUEST)
+
+        self.assertRaisesRegex(exc.BadRequest, 'test error msg',
+                               client.json_request,
+                               'GET', '/v1/resources')
+
     def test_server_https_request_ok(self):
         client = http.HTTPClient('https://localhost/')
-        client.session = utils.FakeSession(
+        client.session = utils.mockSession(
             {'Content-Type': 'application/json'},
-            six.StringIO("Body"),
+            "Body",
             version=1,
             status_code=http_client.OK)
 
@@ -245,9 +437,9 @@ class HttpClientTest(utils.BaseTestCase):
         error_body = _get_error_body()
 
         client = http.HTTPClient('https://localhost/')
-        client.session = utils.FakeSession(
+        client.session = utils.mockSession(
             {'Content-Type': 'application/json'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.INTERNAL_SERVER_ERROR)
 
@@ -258,9 +450,9 @@ class HttpClientTest(utils.BaseTestCase):
     def test_401_unauthorized_exception(self):
         error_body = _get_error_body()
         client = http.HTTPClient('http://localhost/')
-        client.session = utils.FakeSession(
+        client.session = utils.mockSession(
             {'Content-Type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.UNAUTHORIZED)
 
@@ -281,12 +473,12 @@ class HttpClientTest(utils.BaseTestCase):
         expected_result = ('1.1', '1.6')
 
         client = http.HTTPClient('http://localhost/')
-        fake_resp = utils.FakeSessionResponse(
+        fake_resp = utils.mockSessionResponse(
             {'X-OpenStack-Ironic-API-Minimum-Version': '1.1',
              'X-OpenStack-Ironic-API-Maximum-Version': '1.6',
              'Content-Type': 'text/plain',
              },
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.NOT_ACCEPTABLE)
         result = client._parse_version_headers(fake_resp)
@@ -299,12 +491,12 @@ class HttpClientTest(utils.BaseTestCase):
         error_body = _get_error_body()
 
         client = http.HTTPClient('http://%s:%s/' % (host, port))
-        client.session = utils.FakeSession(
+        client.session = utils.mockSession(
             {'X-OpenStack-Ironic-API-Minimum-Version': '1.1',
              'X-OpenStack-Ironic-API-Maximum-Version': latest_ver,
              'content-type': 'text/plain',
              },
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.NOT_ACCEPTABLE)
         self.assertRaises(
@@ -321,20 +513,20 @@ class HttpClientTest(utils.BaseTestCase):
         # Test when fallback to a supported version succeeds
         mock_negotiate.return_value = '1.6'
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'X-OpenStack-Ironic-API-Minimum-Version': '1.1',
              'X-OpenStack-Ironic-API-Maximum-Version': '1.6',
              'content-type': 'text/plain',
              },
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.NOT_ACCEPTABLE)
-        good_resp = utils.FakeSessionResponse(
+        good_resp = utils.mockSessionResponse(
             {'X-OpenStack-Ironic-API-Minimum-Version': '1.1',
              'X-OpenStack-Ironic-API-Maximum-Version': '1.6',
              'content-type': 'text/plain',
              },
-            six.StringIO("We got some text"),
+            "We got some text",
             version=1,
             status_code=http_client.OK)
         client = http.HTTPClient('http://localhost/')
@@ -348,14 +540,33 @@ class HttpClientTest(utils.BaseTestCase):
         self.assertEqual(http_client.OK, response.status_code)
         self.assertEqual(1, mock_negotiate.call_count)
 
+    @mock.patch.object(requests.Session, 'request', autospec=True)
+    @mock.patch.object(http.VersionNegotiationMixin, 'negotiate_version',
+                       autospec=False)
+    def test__http_request_explicit_version(self, mock_negotiate,
+                                            mock_session):
+        headers = {'User-Agent': 'python-ironicclient',
+                   'X-OpenStack-Ironic-API-Version': '1.28'}
+        kwargs = {'os_ironic_api_version': '1.30',
+                  'api_version_select_state': 'negotiated'}
+        mock_session.return_value = utils.mockSessionResponse(
+            {}, status_code=http_client.NO_CONTENT, version=1)
+        client = http.HTTPClient('http://localhost/', **kwargs)
+        response, body_iter = client._http_request('/v1/resources', 'GET',
+                                                   headers=headers)
+        mock_session.assert_called_once_with(mock.ANY, 'GET',
+                                             'http://localhost/v1/resources',
+                                             headers=headers)
+
     @mock.patch.object(http.LOG, 'debug', autospec=True)
     def test_log_curl_request_mask_password(self, mock_log):
         client = http.HTTPClient('http://localhost/')
         kwargs = {'headers': {'foo-header': 'bar-header'},
                   'body': '{"password": "foo"}'}
-        client.log_curl_request('foo', 'http://127.0.0.1', kwargs)
+        client.log_curl_request('foo', '/v1/nodes', kwargs)
         expected_log = ("curl -i -X foo -H 'foo-header: bar-header' "
-                        "-d '{\"password\": \"***\"}' http://127.0.0.1")
+                        "-d '{\"password\": \"***\"}' "
+                        "http://localhost/v1/nodes")
         mock_log.assert_called_once_with(expected_log)
 
     @mock.patch.object(http.LOG, 'debug', autospec=True)
@@ -429,10 +640,17 @@ class HttpClientTest(utils.BaseTestCase):
 
 class SessionClientTest(utils.BaseTestCase):
 
+    @mock.patch.object(http.LOG, 'warning', autospec=True)
+    def test_session_client_endpoint_deprecation(self, log_mock):
+        http.SessionClient(os_ironic_api_version=1, session=mock.Mock(),
+                           api_version_select_state='user', max_retries=5,
+                           retry_interval=5, endpoint='abc')
+        self.assertIn('deprecated', log_mock.call_args[0][0])
+
     def test_server_exception_empty_body(self):
         error_body = _get_error_body()
 
-        fake_session = utils.FakeSession({'Content-Type': 'application/json'},
+        fake_session = utils.mockSession({'Content-Type': 'application/json'},
                                          error_body,
                                          http_client.INTERNAL_SERVER_ERROR)
 
@@ -442,9 +660,21 @@ class SessionClientTest(utils.BaseTestCase):
                           client.json_request,
                           'GET', '/v1/resources')
 
+    def test_server_exception_description_only(self):
+        error_msg = 'test error msg'
+        error_body = _get_error_body(description=error_msg)
+        fake_session = utils.mockSession(
+            {'Content-Type': 'application/json'},
+            error_body, status_code=http_client.BAD_REQUEST)
+        client = _session_client(session=fake_session)
+
+        self.assertRaisesRegex(exc.BadRequest, 'test error msg',
+                               client.json_request,
+                               'GET', '/v1/resources')
+
     def test__parse_version_headers(self):
         # Test parsing of version headers from SessionClient
-        fake_session = utils.FakeSession(
+        fake_session = utils.mockSession(
             {'X-OpenStack-Ironic-API-Minimum-Version': '1.1',
              'X-OpenStack-Ironic-API-Maximum-Version': '1.6',
              'content-type': 'text/plain',
@@ -453,15 +683,15 @@ class SessionClientTest(utils.BaseTestCase):
             http_client.HTTP_VERSION_NOT_SUPPORTED)
         expected_result = ('1.1', '1.6')
         client = _session_client(session=fake_session)
-        result = client._parse_version_headers(fake_session)
+        result = client._parse_version_headers(fake_session.request())
         self.assertEqual(expected_result, result)
 
     def _test_endpoint_override(self, endpoint):
-        fake_session = utils.FakeSession({'content-type': 'application/json'},
+        fake_session = utils.mockSession({'content-type': 'application/json'},
                                          status_code=http_client.NO_CONTENT)
         request_mock = mock.Mock()
         fake_session.request = request_mock
-        request_mock.return_value = utils.FakeSessionResponse(
+        request_mock.return_value = utils.mockSessionResponse(
             headers={'content-type': 'application/json'},
             status_code=http_client.NO_CONTENT)
         client = _session_client(session=fake_session,
@@ -497,15 +727,32 @@ class SessionClientTest(utils.BaseTestCase):
     def test_endpoint_override_not_valid(self):
         self._test_endpoint_override(True)
 
+    def test_make_simple_request(self):
+        session = mock.Mock(spec=['request'])
+
+        client = _session_client(session=session,
+                                 endpoint_override='http://127.0.0.1')
+        res = client._make_simple_request(session, 'GET', 'url')
+
+        session.request.assert_called_once_with(
+            'url', 'GET', raise_exc=False,
+            endpoint_filter={
+                'interface': 'publicURL',
+                'service_type': 'baremetal',
+                'region_name': ''
+            },
+            user_agent=http.USER_AGENT)
+        self.assertEqual(res, session.request.return_value)
+
 
 @mock.patch.object(time, 'sleep', lambda *_: None)
 class RetriesTestCase(utils.BaseTestCase):
 
     def test_http_no_retry(self):
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'Content-Type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.CONFLICT)
         client = http.HTTPClient('http://localhost/', max_retries=0)
@@ -519,14 +766,14 @@ class RetriesTestCase(utils.BaseTestCase):
 
     def test_http_retry(self):
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'Content-Type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.CONFLICT)
-        good_resp = utils.FakeSessionResponse(
+        good_resp = utils.mockSessionResponse(
             {'Content-Type': 'text/plain'},
-            six.StringIO("meow"),
+            "meow",
             version=1,
             status_code=http_client.OK)
         client = http.HTTPClient('http://localhost/')
@@ -542,14 +789,14 @@ class RetriesTestCase(utils.BaseTestCase):
 
     def test_http_retry_503(self):
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'Content-Type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.SERVICE_UNAVAILABLE)
-        good_resp = utils.FakeSessionResponse(
+        good_resp = utils.mockSessionResponse(
             {'Content-Type': 'text/plain'},
-            six.StringIO("meow"),
+            "meow",
             version=1,
             status_code=http_client.OK)
         client = http.HTTPClient('http://localhost/')
@@ -563,9 +810,9 @@ class RetriesTestCase(utils.BaseTestCase):
         self.assertEqual(2, mock_session.request.call_count)
 
     def test_http_retry_connection_refused(self):
-        good_resp = utils.FakeSessionResponse(
+        good_resp = utils.mockSessionResponse(
             {'content-type': 'text/plain'},
-            six.StringIO("meow"),
+            "meow",
             version=1,
             status_code=http_client.OK)
         client = http.HTTPClient('http://localhost/')
@@ -581,9 +828,9 @@ class RetriesTestCase(utils.BaseTestCase):
 
     def test_http_failed_retry(self):
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'content-type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.CONFLICT)
         client = http.HTTPClient('http://localhost/')
@@ -598,9 +845,9 @@ class RetriesTestCase(utils.BaseTestCase):
 
     def test_http_max_retries_none(self):
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'content-type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.CONFLICT)
         client = http.HTTPClient('http://localhost/', max_retries=None)
@@ -615,9 +862,9 @@ class RetriesTestCase(utils.BaseTestCase):
 
     def test_http_change_max_retries(self):
         error_body = _get_error_body()
-        bad_resp = utils.FakeSessionResponse(
+        bad_resp = utils.mockSessionResponse(
             {'content-type': 'text/plain'},
-            six.StringIO(error_body),
+            error_body,
             version=1,
             status_code=http_client.CONFLICT)
         client = http.HTTPClient('http://localhost/',
@@ -634,15 +881,15 @@ class RetriesTestCase(utils.BaseTestCase):
     def test_session_retry(self):
         error_body = _get_error_body()
 
-        fake_resp = utils.FakeSessionResponse(
+        fake_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             error_body,
             http_client.CONFLICT)
-        ok_resp = utils.FakeSessionResponse(
+        ok_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             b"OK",
             http_client.OK)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.side_effect = iter((fake_resp, ok_resp))
 
         client = _session_client(session=fake_session)
@@ -652,15 +899,15 @@ class RetriesTestCase(utils.BaseTestCase):
     def test_session_retry_503(self):
         error_body = _get_error_body()
 
-        fake_resp = utils.FakeSessionResponse(
+        fake_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             error_body,
             http_client.SERVICE_UNAVAILABLE)
-        ok_resp = utils.FakeSessionResponse(
+        ok_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             b"OK",
             http_client.OK)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.side_effect = iter((fake_resp, ok_resp))
 
         client = _session_client(session=fake_session)
@@ -668,11 +915,11 @@ class RetriesTestCase(utils.BaseTestCase):
         self.assertEqual(2, fake_session.request.call_count)
 
     def test_session_retry_connection_refused(self):
-        ok_resp = utils.FakeSessionResponse(
+        ok_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             b"OK",
             http_client.OK)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.side_effect = iter((exc.ConnectionRefused(),
                                                  ok_resp))
 
@@ -681,11 +928,11 @@ class RetriesTestCase(utils.BaseTestCase):
         self.assertEqual(2, fake_session.request.call_count)
 
     def test_session_retry_retriable_connection_failure(self):
-        ok_resp = utils.FakeSessionResponse(
+        ok_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             b"OK",
             http_client.OK)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.side_effect = iter(
             (kexc.RetriableConnectionFailure(), ok_resp))
 
@@ -696,11 +943,11 @@ class RetriesTestCase(utils.BaseTestCase):
     def test_session_retry_fail(self):
         error_body = _get_error_body()
 
-        fake_resp = utils.FakeSessionResponse(
+        fake_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             error_body,
             http_client.CONFLICT)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.return_value = fake_resp
 
         client = _session_client(session=fake_session)
@@ -713,11 +960,11 @@ class RetriesTestCase(utils.BaseTestCase):
     def test_session_max_retries_none(self):
         error_body = _get_error_body()
 
-        fake_resp = utils.FakeSessionResponse(
+        fake_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             error_body,
             http_client.CONFLICT)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.return_value = fake_resp
 
         client = _session_client(session=fake_session)
@@ -731,11 +978,11 @@ class RetriesTestCase(utils.BaseTestCase):
     def test_session_change_max_retries(self):
         error_body = _get_error_body()
 
-        fake_resp = utils.FakeSessionResponse(
+        fake_resp = utils.mockSessionResponse(
             {'Content-Type': 'application/json'},
             error_body,
             http_client.CONFLICT)
-        fake_session = mock.Mock(spec=utils.FakeSession)
+        fake_session = mock.Mock(spec=requests.Session)
         fake_session.request.return_value = fake_resp
 
         client = _session_client(session=fake_session)
