@@ -72,6 +72,7 @@ class TestChassisCreate(TestChassis):
 
         collist = copy.copy(self.collist)
         self.assertEqual(collist, columns)
+        self.assertNotIn('nodes', columns)
 
         datalist = copy.copy(self.datalist)
         self.assertEqual(datalist, tuple(data))
@@ -263,8 +264,7 @@ class TestChassisList(TestChassis):
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
@@ -289,7 +289,7 @@ class TestChassisList(TestChassis):
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         # Set expected values
         kwargs = {
@@ -352,6 +352,15 @@ class TestChassisSet(TestChassis):
                           self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_chassis_set_no_property(self):
+        uuid = baremetal_fakes.baremetal_chassis_uuid
+        arglist = [uuid]
+        verifylist = [('chassis', uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.assertFalse(self.baremetal_mock.chassis.update.called)
+
     def test_chassis_set_description(self):
         description = 'new description'
         uuid = baremetal_fakes.baremetal_chassis_uuid
@@ -431,6 +440,7 @@ class TestChassisShow(TestChassis):
             'uuid'
         )
         self.assertEqual(collist, columns)
+        self.assertNotIn('nodes', columns)
         datalist = (
             baremetal_fakes.baremetal_chassis_description,
             baremetal_fakes.baremetal_chassis_extra,
@@ -459,8 +469,7 @@ class TestChassisShow(TestChassis):
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # DisplayCommandBase.take_action() returns two tuples
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         # Set expected values
         args = [uuid]
@@ -484,7 +493,7 @@ class TestChassisShow(TestChassis):
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         # Set expected values
         args = [uuid]
@@ -532,6 +541,15 @@ class TestChassisUnset(TestChassis):
                           self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_chassis_unset_no_property(self):
+        uuid = baremetal_fakes.baremetal_chassis_uuid
+        arglist = [uuid]
+        verifylist = [('chassis', uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.assertFalse(self.baremetal_mock.chassis.update.called)
+
     def test_chassis_unset_description(self):
         uuid = baremetal_fakes.baremetal_chassis_uuid
         arglist = [
