@@ -61,6 +61,82 @@ class TestCreateBaremetalPort(TestBaremetalPort):
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
+        self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        args = {
+            'address': baremetal_fakes.baremetal_port_address,
+            'node_uuid': baremetal_fakes.baremetal_uuid,
+        }
+
+        self.baremetal_mock.port.create.assert_called_once_with(**args)
+
+    def test_baremetal_port_create_extras(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_address,
+            '--node', baremetal_fakes.baremetal_uuid,
+            '--extra', 'key1=value1',
+            '--extra', 'key2=value2'
+        ]
+
+        verifylist = [
+            ('node_uuid', baremetal_fakes.baremetal_uuid),
+            ('address', baremetal_fakes.baremetal_port_address),
+            ('extra', ['key1=value1', 'key2=value2'])
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        args = {
+            'address': baremetal_fakes.baremetal_port_address,
+            'node_uuid': baremetal_fakes.baremetal_uuid,
+            'extra': baremetal_fakes.baremetal_port_extra
+        }
+        self.baremetal_mock.port.create.assert_called_once_with(**args)
+
+    def test_baremetal_port_create_no_address(self):
+        arglist = ['--node', baremetal_fakes.baremetal_uuid]
+
+        verifylist = [('node_uuid', baremetal_fakes.baremetal_uuid)]
+        self.assertRaises(osctestutils.ParserException,
+                          self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+    def test_baremetal_port_create_no_node(self):
+        arglist = [baremetal_fakes.baremetal_port_address]
+
+        verifylist = [
+            ('address', baremetal_fakes.baremetal_port_address)
+        ]
+        self.assertRaises(osctestutils.ParserException,
+                          self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+    def test_baremetal_port_create_no_args(self):
+        arglist = []
+        verifylist = []
+
+        self.assertRaises(osctestutils.ParserException,
+                          self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+    def test_baremetal_port_create_uuid(self):
+        port_uuid = "da6c8d2e-fbcd-457a-b2a7-cc5c775933af"
+        arglist = [
+            baremetal_fakes.baremetal_port_address,
+            '--node', baremetal_fakes.baremetal_uuid,
+            '--uuid', port_uuid
+        ]
+
+        verifylist = [
+            ('node_uuid', baremetal_fakes.baremetal_uuid),
+            ('address', baremetal_fakes.baremetal_port_address),
+            ('uuid', port_uuid)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
         # DisplayCommandBase.take_action() returns two tuples
         self.cmd.take_action(parsed_args)
 
@@ -68,6 +144,7 @@ class TestCreateBaremetalPort(TestBaremetalPort):
         args = {
             'address': baremetal_fakes.baremetal_port_address,
             'node_uuid': baremetal_fakes.baremetal_uuid,
+            'uuid': port_uuid
         }
 
         self.baremetal_mock.port.create.assert_called_once_with(**args)
@@ -90,8 +167,7 @@ class TestCreateBaremetalPort(TestBaremetalPort):
 
         self.cmd.log = mock.Mock()
 
-        # DisplayCommandBase.take_action() returns two tuples
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         # Set expected values
         args = {
@@ -122,6 +198,62 @@ class TestCreateBaremetalPort(TestBaremetalPort):
                                        'switch_id=aa:bb:cc:dd:ee:ff'])]
         )
 
+    def test_baremetal_port_create_portgroup_uuid(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_address,
+            '--node', baremetal_fakes.baremetal_uuid,
+            '--port-group', baremetal_fakes.baremetal_portgroup_uuid,
+        ]
+
+        verifylist = [
+            ('node_uuid', baremetal_fakes.baremetal_uuid),
+            ('address', baremetal_fakes.baremetal_port_address),
+            ('portgroup_uuid', baremetal_fakes.baremetal_portgroup_uuid)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        args = {
+            'address': baremetal_fakes.baremetal_port_address,
+            'node_uuid': baremetal_fakes.baremetal_uuid,
+            'portgroup_uuid': baremetal_fakes.baremetal_portgroup_uuid
+        }
+
+        self.baremetal_mock.port.create.assert_called_once_with(**args)
+
+    def test_baremetal_port_create_physical_network(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_address,
+            '--node', baremetal_fakes.baremetal_uuid,
+            '--physical-network',
+            baremetal_fakes.baremetal_port_physical_network,
+        ]
+
+        verifylist = [
+            ('node_uuid', baremetal_fakes.baremetal_uuid),
+            ('address', baremetal_fakes.baremetal_port_address),
+            ('physical_network',
+             baremetal_fakes.baremetal_port_physical_network)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        args = {
+            'address': baremetal_fakes.baremetal_port_address,
+            'node_uuid': baremetal_fakes.baremetal_uuid,
+            'physical_network': baremetal_fakes.baremetal_port_physical_network
+        }
+
+        self.baremetal_mock.port.create.assert_called_once_with(**args)
+
 
 class TestShowBaremetalPort(TestBaremetalPort):
     def setUp(self):
@@ -206,6 +338,14 @@ class TestBaremetalPortUnset(TestBaremetalPort):
                           self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_baremetal_port_unset_no_property(self):
+        arglist = [baremetal_fakes.baremetal_port_uuid]
+        verifylist = [('port', baremetal_fakes.baremetal_port_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.assertFalse(self.baremetal_mock.port.update.called)
+
     def test_baremetal_port_unset_extra(self):
         arglist = ['port', '--extra', 'foo']
         verifylist = [('port', 'port'),
@@ -233,6 +373,30 @@ class TestBaremetalPortUnset(TestBaremetalPort):
             [{'path': '/extra/foo', 'op': 'remove'},
              {'path': '/extra/bar', 'op': 'remove'}])
 
+    def test_baremetal_port_unset_portgroup_uuid(self):
+        arglist = ['port', '--port-group']
+        verifylist = [('port', 'port'),
+                      ('portgroup', True)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            'port',
+            [{'path': '/portgroup_uuid', 'op': 'remove'}])
+
+    def test_baremetal_port_unset_physical_network(self):
+        arglist = ['port', '--physical-network']
+        verifylist = [('port', 'port'),
+                      ('physical_network', True)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            'port',
+            [{'path': '/physical_network', 'op': 'remove'}])
+
 
 class TestBaremetalPortSet(TestBaremetalPort):
     def setUp(self):
@@ -291,6 +455,85 @@ class TestBaremetalPortSet(TestBaremetalPort):
             'port',
             [{'path': '/extra/foo', 'value': 'bar', 'op': 'add'}])
 
+    def test_baremetal_port_set_portgroup_uuid(self):
+        new_portgroup_uuid = '1111-111111-1111'
+        arglist = [
+            baremetal_fakes.baremetal_port_uuid,
+            '--port-group', new_portgroup_uuid]
+        verifylist = [
+            ('port', baremetal_fakes.baremetal_port_uuid),
+            ('portgroup_uuid', new_portgroup_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            baremetal_fakes.baremetal_port_uuid,
+            [{'path': '/portgroup_uuid', 'value': new_portgroup_uuid,
+              'op': 'add'}])
+
+    def test_baremetal_set_local_link_connection(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_uuid,
+            '--local-link-connection', 'switch_info=bar']
+        verifylist = [('port', baremetal_fakes.baremetal_port_uuid),
+                      ('local_link_connection', ['switch_info=bar'])]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            baremetal_fakes.baremetal_port_uuid,
+            [{'path': '/local_link_connection/switch_info', 'value': 'bar',
+              'op': 'add'}])
+
+    def test_baremetal_port_set_pxe_enabled(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_uuid,
+            '--pxe-enabled']
+        verifylist = [
+            ('port', baremetal_fakes.baremetal_port_uuid),
+            ('pxe_enabled', True)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            baremetal_fakes.baremetal_port_uuid,
+            [{'path': '/pxe_enabled', 'value': 'True', 'op': 'add'}])
+
+    def test_baremetal_port_set_pxe_disabled(self):
+        arglist = [
+            baremetal_fakes.baremetal_port_uuid,
+            '--pxe-disabled']
+        verifylist = [
+            ('port', baremetal_fakes.baremetal_port_uuid),
+            ('pxe_enabled', False)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            baremetal_fakes.baremetal_port_uuid,
+            [{'path': '/pxe_enabled', 'value': 'False', 'op': 'add'}])
+
+    def test_baremetal_port_set_physical_network(self):
+        new_physical_network = 'physnet2'
+        arglist = [
+            baremetal_fakes.baremetal_port_uuid,
+            '--physical-network', new_physical_network]
+        verifylist = [
+            ('port', baremetal_fakes.baremetal_port_uuid),
+            ('physical_network', new_physical_network)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.baremetal_mock.port.update.assert_called_once_with(
+            baremetal_fakes.baremetal_port_uuid,
+            [{'path': '/physical_network', 'value': new_physical_network,
+              'op': 'add'}])
+
     def test_baremetal_port_set_no_options(self):
         arglist = []
         verifylist = []
@@ -298,6 +541,14 @@ class TestBaremetalPortSet(TestBaremetalPort):
                           self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_baremetal_port_set_no_property(self):
+        arglist = [baremetal_fakes.baremetal_port_uuid]
+        verifylist = [('port', baremetal_fakes.baremetal_port_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.assertFalse(self.baremetal_mock.port.update.called)
+
 
 class TestBaremetalPortDelete(TestBaremetalPort):
     def setUp(self):
@@ -398,7 +649,7 @@ class TestBaremetalPortList(TestBaremetalPort):
         verifylist = [('address', baremetal_fakes.baremetal_port_address)]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         kwargs = {
             'address': baremetal_fakes.baremetal_port_address,
@@ -412,7 +663,7 @@ class TestBaremetalPortList(TestBaremetalPort):
         verifylist = [('node', baremetal_fakes.baremetal_uuid)]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        columns, data = self.cmd.take_action(parsed_args)
+        self.cmd.take_action(parsed_args)
 
         kwargs = {
             'node': baremetal_fakes.baremetal_uuid,
@@ -421,6 +672,21 @@ class TestBaremetalPortList(TestBaremetalPort):
         }
         self.baremetal_mock.port.list.assert_called_with(**kwargs)
 
+    def test_baremetal_port_list_portgroup(self):
+        arglist = ['--port-group', baremetal_fakes.baremetal_portgroup_uuid]
+        verifylist = [('portgroup', baremetal_fakes.baremetal_portgroup_uuid)]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        # DisplayCommandBase.take_action() returns two tuples
+        self.cmd.take_action(parsed_args)
+
+        kwargs = {
+            'portgroup': baremetal_fakes.baremetal_portgroup_uuid,
+            'marker': None,
+            'limit': None,
+        }
+        self.baremetal_mock.port.list.assert_called_with(**kwargs)
+
     def test_baremetal_port_list_long(self):
         arglist = ['--long']
         verifylist = [('detail', True)]
@@ -436,7 +702,8 @@ class TestBaremetalPortList(TestBaremetalPort):
         self.baremetal_mock.port.list.assert_called_with(**kwargs)
 
         collist = ('UUID', 'Address', 'Created At', 'Extra', 'Node UUID',
-                   'Local Link Connection', 'PXE boot enabled', 'Updated At',
+                   'Local Link Connection', 'Portgroup UUID',
+                   'PXE boot enabled', 'Physical Network', 'Updated At',
                    'Internal Info')
         self.assertEqual(collist, columns)
 
@@ -449,6 +716,8 @@ class TestBaremetalPortList(TestBaremetalPort):
             '',
             '',
             '',
+            '',
+            '',
             ''
         ), )
         self.assertEqual(datalist, tuple(data))
