@@ -12,19 +12,19 @@
 
 import re
 import sys
-import uuid
 
 import fixtures
-import httplib2
 from keystoneauth1 import exceptions as keystone_exc
 from keystoneauth1 import fixture as ks_fixture
 import mock
+from oslo_utils import uuidutils
 import requests_mock
 import six
 import testtools
 from testtools import matchers
 
 from ironicclient import client
+from ironicclient.common.apiclient import exceptions
 from ironicclient.common import http
 from ironicclient import exc
 from ironicclient import shell as ironic_shell
@@ -36,20 +36,27 @@ V3_URL = BASE_URL + '/v3'
 
 FAKE_ENV = {'OS_USERNAME': 'username',
             'OS_PASSWORD': 'password',
-            'OS_TENANT_NAME': 'tenant_name',
+            'OS_PROJECT_NAME': 'project_name',
             'OS_AUTH_URL': V2_URL}
 
+FAKE_ENV_WITH_SSL = FAKE_ENV.copy()
+FAKE_ENV_WITH_SSL.update({
+    'OS_CACERT': 'cacert',
+    'OS_CERT': 'cert',
+    'OS_KEY': 'key',
+})
+
 FAKE_ENV_KEYSTONE_V2 = {
     'OS_USERNAME': 'username',
     'OS_PASSWORD': 'password',
-    'OS_TENANT_NAME': 'tenant_name',
+    'OS_PROJECT_NAME': 'project_name',
     'OS_AUTH_URL': V2_URL
 }
 
 FAKE_ENV_KEYSTONE_V3 = {
     'OS_USERNAME': 'username',
     'OS_PASSWORD': 'password',
-    'OS_TENANT_NAME': 'tenant_name',
+    'OS_PROJECT_NAME': 'project_name',
     'OS_AUTH_URL': V3_URL,
     'OS_USER_DOMAIN_ID': 'default',
     'OS_PROJECT_DOMAIN_ID': 'default',
@@ -57,7 +64,7 @@ FAKE_ENV_KEYSTONE_V3 = {
 
 FAKE_ENV_KEYSTONE_V2_TOKEN = {
     'OS_AUTH_TOKEN': 'admin_token',
-    'OS_TENANT_NAME': 'tenant_name',
+    'OS_PROJECT_NAME': 'project_name',
     'OS_AUTH_URL': V2_URL
 }
 
@@ -74,28 +81,22 @@ class ShellTest(utils.BaseTestCase):
         super(ShellTest, self).setUp()
 
     def shell(self, argstr):
-        orig = sys.stdout
-        try:
-            sys.stdout = six.StringIO()
-            _shell = ironic_shell.IronicShell()
-            _shell.main(argstr.split())
-        except SystemExit:
-            exc_type, exc_value, exc_traceback = sys.exc_info()
-            self.assertEqual(0, exc_value.code)
-        finally:
-            out = sys.stdout.getvalue()
-            sys.stdout.close()
-            sys.stdout = orig
-        return out
+        with mock.patch.object(sys, 'stdout', six.StringIO()):
+            with mock.patch.object(sys, 'stderr', six.StringIO()):
+                try:
+                    _shell = ironic_shell.IronicShell()
+                    _shell.main(argstr.split())
+                except SystemExit:
+                    exc_type, exc_value, exc_traceback = sys.exc_info()
+                    self.assertEqual(0, exc_value.code)
+                finally:
+                    out = sys.stdout.getvalue()
+                    err = sys.stderr.getvalue()
+                return out, err
 
     def test_help_unknown_command(self):
         self.assertRaises(exc.CommandError, self.shell, 'help foofoo')
 
-    def test_debug(self):
-        httplib2.debuglevel = 0
-        self.shell('--debug help')
-        self.assertEqual(1, httplib2.debuglevel)
-
     def test_help(self):
         required = [
             '.*?^usage: ironic',
@@ -104,7 +105,7 @@ class ShellTest(utils.BaseTestCase):
             'for help on a specific command',
         ]
         for argstr in ['--help', 'help']:
-            help_text = self.shell(argstr)
+            help_text = self.shell(argstr)[0]
             for r in required:
                 self.assertThat(help_text,
                                 matchers.MatchesRegex(r,
@@ -112,14 +113,44 @@ class ShellTest(utils.BaseTestCase):
 
     def test_help_on_subcommand(self):
         required = [
-            '.*?^usage: ironic chassis-show',
+            ".*?^usage: ironic chassis-show",
             ".*?^Show detailed information about a chassis",
         ]
         argstrings = [
             'help chassis-show',
         ]
         for argstr in argstrings:
-            help_text = self.shell(argstr)
+            help_text = self.shell(argstr)[0]
+            for r in required:
+                self.assertThat(help_text,
+                                matchers.MatchesRegex(r, self.re_options))
+
+    def test_required_args_on_node_create_help(self):
+        required = [
+            ".*?^usage: ironic node-create",
+            ".*?^Register a new node with the Ironic service",
+            ".*?^Required arguments:",
+        ]
+        argstrings = [
+            'help node-create',
+        ]
+        for argstr in argstrings:
+            help_text = self.shell(argstr)[0]
+            for r in required:
+                self.assertThat(help_text,
+                                matchers.MatchesRegex(r, self.re_options))
+
+    def test_required_args_on_port_create_help(self):
+        required = [
+            ".*?^usage: ironic port-create",
+            ".*?^Create a new port",
+            ".*?^Required arguments:",
+        ]
+        argstrings = [
+            'help port-create',
+        ]
+        for argstr in argstrings:
+            help_text = self.shell(argstr)[0]
             for r in required:
                 self.assertThat(help_text,
                                 matchers.MatchesRegex(r, self.re_options))
@@ -128,62 +159,112 @@ class ShellTest(utils.BaseTestCase):
         self.make_env(exclude='OS_USERNAME')
         self.test_help()
 
-    @mock.patch.object(client, 'get_client',
+    @mock.patch.object(client, 'get_client', autospec=True,
                        side_effect=keystone_exc.ConnectFailure)
-    @mock.patch('sys.stdin', side_effect=mock.MagicMock)
-    @mock.patch('getpass.getpass', return_value='password')
+    @mock.patch('sys.stdin', side_effect=mock.MagicMock, autospec=True)
+    @mock.patch('getpass.getpass', return_value='password', autospec=True)
     def test_password_prompted(self, mock_getpass, mock_stdin, mock_client):
         self.make_env(exclude='OS_PASSWORD')
         # We will get a ConnectFailure because there is no keystone.
         self.assertRaises(keystone_exc.ConnectFailure,
                           self.shell, 'node-list')
         expected_kwargs = {
-            'ironic_url': '', 'os_auth_url': FAKE_ENV['OS_AUTH_URL'],
-            'os_tenant_id': '', 'os_tenant_name': FAKE_ENV['OS_TENANT_NAME'],
-            'os_username': FAKE_ENV['OS_USERNAME'], 'os_user_domain_id': '',
-            'os_user_domain_name': '', 'os_password': FAKE_ENV['OS_PASSWORD'],
-            'os_auth_token': '', 'os_project_id': '',
-            'os_project_name': '', 'os_project_domain_id': '',
-            'os_project_domain_name': '', 'os_region_name': '',
-            'os_service_type': '', 'os_endpoint_type': '', 'os_cacert': None,
-            'os_cert': None, 'os_key': None,
+            'auth_url': FAKE_ENV['OS_AUTH_URL'],
+            'username': FAKE_ENV['OS_USERNAME'],
+            'password': FAKE_ENV['OS_PASSWORD'],
+            'project_name': FAKE_ENV['OS_PROJECT_NAME'],
             'max_retries': http.DEFAULT_MAX_RETRIES,
             'retry_interval': http.DEFAULT_RETRY_INTERVAL,
-            'os_ironic_api_version': None, 'timeout': 600, 'insecure': False
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'timeout': 600, 'insecure': False
         }
         mock_client.assert_called_once_with(1, **expected_kwargs)
         # Make sure we are actually prompted.
         mock_getpass.assert_called_with('OpenStack Password: ')
 
-    @mock.patch.object(client, 'get_client',
+    @mock.patch.object(client, 'get_client', autospec=True,
                        side_effect=keystone_exc.ConnectFailure)
-    @mock.patch('getpass.getpass', return_value='password')
+    @mock.patch('sys.stdin', side_effect=mock.MagicMock, autospec=True)
+    @mock.patch('getpass.getpass', return_value='password', autospec=True)
+    def test_password(self, mock_getpass, mock_stdin, mock_client):
+        self.make_env(environ_dict=FAKE_ENV_WITH_SSL)
+        # We will get a ConnectFailure because there is no keystone.
+        self.assertRaises(keystone_exc.ConnectFailure,
+                          self.shell, 'node-list')
+        expected_kwargs = {
+            'auth_url': FAKE_ENV_WITH_SSL['OS_AUTH_URL'],
+            'username': FAKE_ENV_WITH_SSL['OS_USERNAME'],
+            'password': FAKE_ENV_WITH_SSL['OS_PASSWORD'],
+            'project_name': FAKE_ENV_WITH_SSL['OS_PROJECT_NAME'],
+            'cafile': FAKE_ENV_WITH_SSL['OS_CACERT'],
+            'certfile': FAKE_ENV_WITH_SSL['OS_CERT'],
+            'keyfile': FAKE_ENV_WITH_SSL['OS_KEY'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'timeout': 600,
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'insecure': False
+        }
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+        self.assertFalse(mock_getpass.called)
+
+    @mock.patch.object(client, 'get_client', autospec=True,
+                       side_effect=keystone_exc.ConnectFailure)
+    @mock.patch('getpass.getpass', return_value='password', autospec=True)
     def test_token_auth(self, mock_getpass, mock_client):
         self.make_env(environ_dict=FAKE_ENV_KEYSTONE_V2_TOKEN)
         # We will get a ConnectFailure because there is no keystone.
         self.assertRaises(keystone_exc.ConnectFailure,
                           self.shell, 'node-list')
         expected_kwargs = {
-            'ironic_url': '',
-            'os_auth_url': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_AUTH_URL'],
-            'os_tenant_id': '',
-            'os_tenant_name': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_TENANT_NAME'],
-            'os_username': '', 'os_user_domain_id': '',
-            'os_user_domain_name': '', 'os_password': '',
-            'os_auth_token': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_AUTH_TOKEN'],
-            'os_project_id': '', 'os_project_name': '',
-            'os_project_domain_id': '', 'os_project_domain_name': '',
-            'os_region_name': '', 'os_service_type': '',
-            'os_endpoint_type': '', 'os_cacert': None, 'os_cert': None,
-            'os_key': None, 'max_retries': http.DEFAULT_MAX_RETRIES,
+            'auth_url': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_AUTH_URL'],
+            'token': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_AUTH_TOKEN'],
+            'project_name': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_PROJECT_NAME'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
             'retry_interval': http.DEFAULT_RETRY_INTERVAL,
-            'os_ironic_api_version': None, 'timeout': 600, 'insecure': False
+            'timeout': 600,
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'insecure': False
         }
         mock_client.assert_called_once_with(1, **expected_kwargs)
         self.assertFalse(mock_getpass.called)
 
-    @mock.patch('sys.stdin', side_effect=mock.MagicMock)
-    @mock.patch('getpass.getpass', side_effect=EOFError)
+    @mock.patch.object(client, 'get_client', autospec=True)
+    @mock.patch('getpass.getpass', return_value='password', autospec=True)
+    def test_admin_token_auth(self, mock_getpass, mock_client):
+        self.make_env(environ_dict={
+            'IRONIC_URL': 'http://192.168.1.1/v1',
+            'OS_AUTH_TOKEN': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_AUTH_TOKEN']})
+        expected_kwargs = {
+            'endpoint': 'http://192.168.1.1/v1',
+            'token': FAKE_ENV_KEYSTONE_V2_TOKEN['OS_AUTH_TOKEN'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'timeout': 600,
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'insecure': False
+        }
+        self.shell('node-list')
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+        self.assertFalse(mock_getpass.called)
+
+    @mock.patch.object(client, 'get_client', autospec=True)
+    @mock.patch('getpass.getpass', return_value='password', autospec=True)
+    def test_none_auth(self, mock_getpass, mock_client):
+        self.make_env(environ_dict={'IRONIC_URL': 'http://192.168.1.1/v1'})
+        expected_kwargs = {
+            'endpoint': 'http://192.168.1.1/v1',
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'timeout': 600, 'insecure': False
+        }
+        self.shell('node-list')
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+        self.assertFalse(mock_getpass.called)
+
+    @mock.patch('sys.stdin', side_effect=mock.MagicMock, autospec=True)
+    @mock.patch('getpass.getpass', side_effect=EOFError, autospec=True)
     def test_password_prompted_ctrlD(self, mock_getpass, mock_stdin):
         self.make_env(exclude='OS_PASSWORD')
         # We should get Command Error because we mock Ctl-D.
@@ -192,7 +273,7 @@ class ShellTest(utils.BaseTestCase):
         # Make sure we are actually prompted.
         mock_getpass.assert_called_with('OpenStack Password: ')
 
-    @mock.patch('sys.stdin')
+    @mock.patch('sys.stdin', autospec=True)
     def test_no_password_no_tty(self, mock_stdin):
         # delete the isatty attribute so that we do not get
         # prompted when manually running the tests
@@ -209,7 +290,7 @@ class ShellTest(utils.BaseTestCase):
             self.fail('CommandError not raised')
 
     def test_bash_completion(self):
-        stdout = self.shell('bash-completion')
+        stdout = self.shell('bash-completion')[0]
         # just check we have some output
         required = [
             '.*--driver_info',
@@ -222,17 +303,107 @@ class ShellTest(utils.BaseTestCase):
                             matchers.MatchesRegex(r, self.re_options))
 
     def test_ironic_api_version(self):
-        self.shell('--ironic-api-version 1.2 help')
-        self.shell('--ironic-api-version latest help')
+        err = self.shell('--ironic-api-version 1.2 help')[1]
+        self.assertIn('The "ironic" CLI is deprecated', err)
+
+        err = self.shell('--ironic-api-version latest help')[1]
+        self.assertIn('The "ironic" CLI is deprecated', err)
+
+        err = self.shell('--ironic-api-version 1 help')[1]
+        self.assertIn('The "ironic" CLI is deprecated', err)
+
+    def test_invalid_ironic_api_version(self):
+        self.assertRaises(exceptions.UnsupportedVersion,
+                          self.shell, '--ironic-api-version 0.8 help')
         self.assertRaises(exc.CommandError,
                           self.shell, '--ironic-api-version 1.2.1 help')
 
+    @mock.patch.object(client, 'get_client', autospec=True,
+                       side_effect=keystone_exc.ConnectFailure)
+    def test_api_version_in_env(self, mock_client):
+        env = dict(IRONIC_API_VERSION='1.10', **FAKE_ENV)
+        self.make_env(environ_dict=env)
+        # We will get a ConnectFailure because there is no keystone.
+        self.assertRaises(keystone_exc.ConnectFailure,
+                          self.shell, 'node-list')
+        expected_kwargs = {
+            'auth_url': FAKE_ENV['OS_AUTH_URL'],
+            'username': FAKE_ENV['OS_USERNAME'],
+            'password': FAKE_ENV['OS_PASSWORD'],
+            'project_name': FAKE_ENV['OS_PROJECT_NAME'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'os_ironic_api_version': '1.10',
+            'timeout': 600, 'insecure': False
+        }
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+
+    @mock.patch.object(client, 'get_client', autospec=True,
+                       side_effect=keystone_exc.ConnectFailure)
+    def test_api_version_v1_in_env(self, mock_client):
+        env = dict(IRONIC_API_VERSION='1', **FAKE_ENV)
+        self.make_env(environ_dict=env)
+        # We will get a ConnectFailure because there is no keystone.
+        self.assertRaises(keystone_exc.ConnectFailure,
+                          self.shell, 'node-list')
+        expected_kwargs = {
+            'auth_url': FAKE_ENV['OS_AUTH_URL'],
+            'username': FAKE_ENV['OS_USERNAME'],
+            'password': FAKE_ENV['OS_PASSWORD'],
+            'project_name': FAKE_ENV['OS_PROJECT_NAME'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'timeout': 600, 'insecure': False
+        }
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+
+    @mock.patch.object(client, 'get_client', autospec=True,
+                       side_effect=keystone_exc.ConnectFailure)
+    def test_api_version_in_args(self, mock_client):
+        env = dict(IRONIC_API_VERSION='1.10', **FAKE_ENV)
+        self.make_env(environ_dict=env)
+        # We will get a ConnectFailure because there is no keystone.
+        self.assertRaises(keystone_exc.ConnectFailure,
+                          self.shell, '--ironic-api-version 1.11 node-list')
+        expected_kwargs = {
+            'auth_url': FAKE_ENV['OS_AUTH_URL'],
+            'username': FAKE_ENV['OS_USERNAME'],
+            'password': FAKE_ENV['OS_PASSWORD'],
+            'project_name': FAKE_ENV['OS_PROJECT_NAME'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'os_ironic_api_version': '1.11',
+            'timeout': 600, 'insecure': False
+        }
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+
+    @mock.patch.object(client, 'get_client', autospec=True,
+                       side_effect=keystone_exc.ConnectFailure)
+    def test_api_version_v1_in_args(self, mock_client):
+        env = dict(IRONIC_API_VERSION='1.10', **FAKE_ENV)
+        self.make_env(environ_dict=env)
+        # We will get a ConnectFailure because there is no keystone.
+        self.assertRaises(keystone_exc.ConnectFailure,
+                          self.shell, '--ironic-api-version 1 node-list')
+        expected_kwargs = {
+            'auth_url': FAKE_ENV['OS_AUTH_URL'],
+            'username': FAKE_ENV['OS_USERNAME'],
+            'password': FAKE_ENV['OS_PASSWORD'],
+            'project_name': FAKE_ENV['OS_PROJECT_NAME'],
+            'max_retries': http.DEFAULT_MAX_RETRIES,
+            'retry_interval': http.DEFAULT_RETRY_INTERVAL,
+            'os_ironic_api_version': ironic_shell.LATEST_VERSION,
+            'timeout': 600, 'insecure': False
+        }
+        mock_client.assert_called_once_with(1, **expected_kwargs)
+
 
 class TestCase(testtools.TestCase):
 
     def set_fake_env(self, fake_env):
-        client_env = ('OS_USERNAME', 'OS_PASSWORD', 'OS_TENANT_ID',
-                      'OS_TENANT_NAME', 'OS_AUTH_URL', 'OS_REGION_NAME',
+        client_env = ('OS_USERNAME', 'OS_PASSWORD', 'OS_PROJECT_ID',
+                      'OS_PROJECT_NAME', 'OS_AUTH_URL', 'OS_REGION_NAME',
                       'OS_AUTH_TOKEN', 'OS_NO_CLIENT_AUTH', 'OS_SERVICE_TYPE',
                       'OS_ENDPOINT_TYPE', 'OS_CACERT', 'OS_CERT', 'OS_KEY')
 
@@ -240,17 +411,6 @@ class TestCase(testtools.TestCase):
             self.useFixture(
                 fixtures.EnvironmentVariable(key, fake_env.get(key)))
 
-    # required for testing with Python 2.6
-    def assertRegexpMatches(self, text, expected_regexp, msg=None):
-        """Fail the test unless the text matches the regular expression."""
-        if isinstance(expected_regexp, six.string_types):
-            expected_regexp = re.compile(expected_regexp)
-        if not expected_regexp.search(text):
-            msg = msg or "Regexp didn't match"
-            msg = '%s: %r not found in %r' % (
-                msg, expected_regexp.pattern, text)
-            raise self.failureException(msg)
-
     def register_keystone_v2_token_fixture(self, request_mocker):
         v2_token = ks_fixture.V2Token()
         service = v2_token.add_service('baremetal')
@@ -265,7 +425,7 @@ class TestCase(testtools.TestCase):
         request_mocker.post(
             '%s/auth/tokens' % V3_URL,
             json=v3_token,
-            headers={'X-Subject-Token': uuid.uuid4().hex})
+            headers={'X-Subject-Token': uuidutils.generate_uuid()})
 
     def register_keystone_auth_fixture(self, request_mocker):
         self.register_keystone_v2_token_fixture(request_mocker)
@@ -276,9 +436,9 @@ class TestCase(testtools.TestCase):
         request_mocker.get(BASE_URL, json=ks_fixture.DiscoveryList(BASE_URL))
 
 
-class ShellTestNoMox(TestCase):
+class ShellTestKeystoneV2(TestCase):
     def setUp(self):
-        super(ShellTestNoMox, self).setUp()
+        super(ShellTestKeystoneV2, self).setUp()
         self.set_fake_env(FAKE_ENV_KEYSTONE_V2)
 
     def shell(self, argstr):
@@ -337,10 +497,10 @@ class ShellTestNoMox(TestCase):
         ]
 
         for r in required:
-            self.assertRegexpMatches(event_list_text, r)
+            self.assertRegex(event_list_text, r)
 
 
-class ShellTestNoMoxV3(ShellTestNoMox):
+class ShellTestKeystoneV3(ShellTestKeystoneV2):
 
     def _set_fake_env(self):
         self.set_fake_env(FAKE_ENV_KEYSTONE_V3)
